/*
     This file is part of GNUnet.
     (C) 2001, 2002 Christian Grothoff (and other contributing authors)

     GNUnet is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published
     by the Free Software Foundation; either version 2, or (at your
     option) any later version.

     GNUnet is distributed in the hope that it will be useful, but
     WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with GNUnet; see the file COPYING.  If not, write to the
     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
*/

/**
 * Wrapper around malloc/free.
 * @file util/xmalloc.c
 * @author Christian Grothoff
 **/ 

#include "config.h"
#include "util/xmalloc.h"
#include "util/logging.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>


#define DEBUG_MALLOC 0

/**
 * Allocate memory. Checks the return value, aborts if no more
 * memory is available.
 **/
void * xmalloc(int size, 
	       const char * site) {
  void * result;

  result = malloc(size);
#if DEBUG_MALLOC > 2
  printf("xmalloc: allocating %d bytes (%x) at site %s\n", 
	 size, 
	 (int) result, 
	 site);
  fflush(stdout);
#endif  
#if DEBUG_MALLOC == 2
  printf("%x malloc %s\n", (int)result, site);
  fflush(stdout);
#endif
  if (result == NULL) {
    errexit("ERROR: out of memory (%d at %s)!\n",
	    size, site);
  }
  memset(result, 0, size); /* client code should not rely on this, though... */
  return result;
}

/**
 * Free memory. Merely a wrapper for the case that we
 * want to keep track of allocations.
 **/
void xfree(void * ptr, 
	   const char * site) {
#if DEBUG_MALLOC
  if (ptr == NULL)
    errexit("xfree called with NULL ptr at %s\n", site);
#if DEBUG_MALLOC > 2
  printf("xfree: %s (%x)\n", 
	 site, 
	 (int) ptr);
  fflush(stdout);
#endif
#if DEBUG_MALLOC == 2
  printf("%x free %s\n", (int) ptr, site);
  fflush(stdout);
#endif
#endif
  free(ptr);
}

/**
 * Free if not NULL.
 **/
void freeNonNull(void * ptr,
		 const char * site) {
  if (ptr != NULL)
    xfree(ptr, site);
}

/**
 * Dup a string.
 **/
char * xstrdup(const char * str,
	       const char * filename,
	       const int linenumber) {
  char * res;

  res = strdup(str);
#if DEBUG_MALLOC == 2
  printf("%x strdup %s:%d\n", 
	 (int) res, 
	 filename,
	 linenumber);
  fflush(stdout);
#endif
  return res;
}

/* end of xmalloc.c */
